/*-----------------------------------------------------------------------

File  : clb_pdarrays.c

Author: Stephan Schulz

Contents
 
  Funktions realising the dynamic array type.

Changes

<1> Thu Jul 23 17:40:32 MEST 1998
    New

-----------------------------------------------------------------------*/

#include "clb_pdarrays.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: PDArrayAlloc()
//
//   Return an initialized dynamic array of size init_size.
//
// Global Variables: -
//
// Side Effects    : Memory Operations
//
/----------------------------------------------------------------------*/

PDArray_p PDArrayAlloc(long init_size, long grow)
{
   PDArray_p handle = PDArrayCellAlloc();
   int i;
   
   assert(init_size > 0);
   assert(grow > 0);

   handle->size  = init_size;
   handle->grow  = grow;
   handle->array = SizeMalloc(handle->size*sizeof(IntOrP));
   for(i=0; i<handle->size; i++)
   {
      handle->array[i].p_val = NULL;
   }
   return handle;
}


/*-----------------------------------------------------------------------
//
// Function: PDArrayFree()
//
//   Free a PDArray. Leaves elements untouched.
//
// Global Variables: -
//
// Side Effects    : Memory Operations
//
/----------------------------------------------------------------------*/

void PDArrayFree(PDArray_p junk)
{
   assert(junk);
   assert(junk->size > 0);
   assert(junk->array);

   SizeFree(junk->array, junk->size*sizeof(IntOrP));
   PDArrayCellFree(junk);
}


/*-----------------------------------------------------------------------
//
// Function: PDArrayElementRef()
//
//   Return a reference to an element in a dynamic array. This
//   reference is only good until the next call to this function! User
//   programs are expected to use this function only extremely rarely
//   and with special care. Use PDArrayElement()/PDArrayAssign()
//   instead. 
//
// Global Variables: -
//
// Side Effects    : May enlarge and move array.
//
/----------------------------------------------------------------------*/

IntOrP* PDArrayElementRef(PDArray_p array, long index)
{
   if(!(index < array->size))
   {
      IntOrP *tmp;
      int    old_size, i;
      
      /* Enlarge array */

      old_size = array->size;
      tmp      = array->array;
      array->size = ((index/array->grow)+1)*array->grow;
      array->array = SizeMalloc(array->size * sizeof(IntOrP));
      memcpy(array->array, tmp, old_size*sizeof(IntOrP));
      SizeFree(tmp, old_size * sizeof(IntOrP));
      for(i=old_size; i<array->size; i++)
      {
	 array->array[i].p_val = NULL;
      }      
   }
   return &(array->array[index]);
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


