/*-----------------------------------------------------------------------

File  : clb_ptrees.h

Author: Stephan Schulz

Contents
 
  Data structures for the efficient management of pointer
  sets. Another incarnation of my beloved AVL trees, but with keys
  only. As pointers are managed, all additional information can go
  into the pointed-to structures.

Changes

<1> Thu Sep 25 02:36:58 MET DST 1997
    New

-----------------------------------------------------------------------*/

#ifndef CLB_PTREES

#define CLB_PTREES

#include <clb_pstacks.h>
#include <clb_avlgeneric.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/


/* Data structure for indexing pointers (which need to be casted
   carefully by the wrapper functions). */ 

typedef struct ptreecell
{
   void*            key;
   short            balance;
   short            passed;
   struct ptreecell *lson;
   struct ptreecell *rson;
}PTreeCell, *PTree_p;



/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define PTreeCellAlloc()    (PTreeCell*)SizeMalloc(sizeof(PTreeCell))
#define PTreeCellFree(junk) SizeFree(junk, sizeof(PTreeCell))

#define PCmp(p1, p2) ((int)(p1)-(int)(p2))
int     PCmpFun(void* p1, void*p2);
PTree_p PTreeCellAllocEmpty();
void    PTreeFree(PTree_p junk);
PTree_p PTreeInsert(PTree_p *root, PTree_p new);
bool    PTreeStore(PTree_p *root, void* key);
PTree_p PTreeFind(PTree_p root, void* key);
PTree_p PTreeExtractEntry(PTree_p *root, void* key);
void*   PTreeExtractKey(PTree_p *root, void* key);
void*   PTreeExtractRootKey(PTree_p *root);
bool    PTreeDeleteEntry(PTree_p *root, void* key);
void    PTreeMerge(PTree_p *root, PTree_p add);
long    PTreeNodes(PTree_p root);

AVL_TRAVERSE_DECLARATION(PTree, PTree_p)
#define PTreeTraverseExit(stack) PStackFree(stack)

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





