/*-----------------------------------------------------------------------

File  : clb_sysdate.h

Author: Stephan Schulz

Contents
 
  Data types dealing with "dates" and "times". A "time" in this
  context is a data type with a defined starting point and a total
  ordering that monotonically increases during the run of the program
  and can be used to define an order of events. A "date" is a specific
  element from a "time".

Changes

<1> Wed Apr  8 16:49:44 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CLB_SYSDATE

#define CLB_SYSDATE

#include <stdio.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

/* Data type used both for "time" keeping and recording of
   "dates". Uses a struct because it may become necessary to extend
   this (2 to the power of 32 is not _really_ big...) */

typedef struct sysdate
{
   unsigned long date;
}SysDate, *SysDate_p;
   

typedef enum 
{
   DateEarlier = -1,
   DateEqual = 0,
   DateLater = 1
}DateRelation;

/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

SysDate      SysDateCreationTime();
#define      SysDateInc(sd) (((sd)->date)++);assert((sd)->date);
DateRelation SysDateCompare(SysDate date1, SysDate date2);
void         SysDatePrint(FILE* out, SysDate date);
#define      SysDateMaximum(date1, date2) \
             (SysDateCompare((date1), (date2))==DateEarlier?\
             (date2):(date1))

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





