/*-----------------------------------------------------------------------

File  : ccl_clauses.h

Author: Stephan Schulz

Contents
 
  Clauses - Infrastructure functions

Changes

<1> Thu Apr 16 19:38:16 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CCL_CLAUSES

#define CCL_CLAUSES

#include <ccl_evaluations.h>
#include <ccl_eqnlist.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/


typedef struct clause_cell
{
   long                  ident;       /* Hopefully unique ident for
					 all clauses created dring
					 proof run */
   SysDate               date;        /* ...at which this clause
					 became a demodulator */			     
   Eqn_p                 literals;    /* List of literals */
   int                   tot_lit_no;  /* Number of literals */
   int                   neg_lit_no;  /* Negative literals */
   int                   pos_lit_no;  /* Positive literals */
   Eval_p                evaluations; /* List of evauations */
   struct clausesetcell* set;         /* Is the clause in a set? */
   bool                  fanned;      /* Has this claused been fanned? */
   bool                  procedural;  /* Should this clause be fanned?
			  	         Is there a reasonable semantics
				         for procedural equational
				         clauses? */		     
   PTree_p               children;    /* Which can be removed if this
			  	         clause changes significantly */
   struct clause_cell*   parent1;     /* Parents need to be notified */
   struct clause_cell*   parent2;     /* if their children are removed! */
   struct clause_cell*   pred;        /* For clause sets = doubly  */
   struct clause_cell*   succ;        /*    linked lists */
}ClauseCell, *Clause_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


#define ClauseCellAlloc()    (ClauseCell*)SizeMalloc(sizeof(ClauseCell))
#define ClauseCellFree(junk) SizeFree(junk, sizeof(ClauseCell))

Clause_p EmptyClauseAlloc();
Clause_p ClauseAlloc(Eqn_p literals);
void     ClauseFree(Clause_p junk);

Clause_p ClauseCopy(Clause_p clause, TB_p bank);
#define  ClauseIsGoal(clause) (!((clause)->pos_lit_no))
#ifdef DEMOD_VAR_RESTRICTION
#define  ClauseIsDemodulator(clause)\
           (((clause)->pos_lit_no == 1) && \
	    ((clause)->neg_lit_no == 0) && \
	    (((clause)->literals->type == EqOriented)|| \
	     ((clause)->literals->demod_sides != NoSide)))
#else
#define  ClauseIsDemodulator(clause)\
            (((clause)->pos_lit_no == 1) && \
	     ((clause)->neg_lit_no == 0))
#endif


#define  ClauseIsTautology(clause) EqnListIsTautologic(clause->literals)
#define  ClauseIsGround(clause) EqnListIsGround(clause->literals)
void     ClausePrintList(FILE* out, Clause_p clause, bool fullterms);
void     ClausePrintAxiom(FILE* out, Clause_p clause, bool fullterms);
void     ClausePrintRule(FILE* out, Clause_p clause, bool fullterms);
void     ClausePrintGoal(FILE* out, Clause_p clause, bool fullterms);
void     ClausePrintQuery(FILE* out, Clause_p clause, bool fullterms);

void     ClausePrint(FILE* out, Clause_p clause, bool fullterms); 
bool     ClauseStartsMaybe(Scanner_p in);
Clause_p ClauseParse(Scanner_p in, TB_p bank);

void     ClauseMarkMaximalTerms(OCB_p ocb, Clause_p clause);
void     ClauseMarkRestrictedTerms(Clause_p clause);

void     ClauseRecordTermDates(Clause_p clause);
void     ClauseDetachParents(Clause_p clause);
void     ClauseRegisterChild(Clause_p clause, Clause_p child);
bool     ClauseRemoveSuperfluousLiterals(Clause_p clause);

void     ClauseAddEvalCell(Clause_p clause, Eval_p evaluation);
void     ClauseRemoveEvaluations(Clause_p clause);

double   ClauseWeight(Clause_p clause, double max_multiplier, double
		      pos_multiplier, long vweight, long fweight);

#define  NormSubstClause(clause, subst, vars)\
         NormSubstEqnListExcept((clause)->literals,\
				NULL, (subst), (vars));

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





