/*-----------------------------------------------------------------------

File  : ccl_eqn.h

Author: Stephan Schulz

Contents
 
  The termpair datatype: Rules, Equations, positive and negative
  literals. 

Changes

<1> Fri Mar 13 17:09:13 MET 1998
    New

-----------------------------------------------------------------------*/

#ifndef CCL_EQN

#define CCL_EQN

#include <cte_match_mgu_1-1.h>
#include <cte_replace.h>
#include <cto_orderings.h>
#include <cto_basefuns.h>



/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef enum 
{
   EqOriented,    /* lterm is bigger than rterm */
   EqUnoriented,  /* Terms have not been compared yet (default) */
   EqUnorientable /* Terms are uncomparable */
}EqnType;


/* Basic data structure for rules, equations, literals. I dropped the
   following idea after thinking a long time...it really is _not_ a
   good idea!

   !!!!! This is outdated !!!!!!!
   !
   !   If rside is NULL, it is assumed to be equal to the special
   !   non-existant (in the implementation) constant TRUE (or
   !   whatever)...I hope you know what I mean, i.e. it is a
   !   non-equational literal that can be considered to be in equational
   !   form. Is this a good idea? I'm thinking about it ;-) ... still
   !   thinking about it....seems to lead to a lot of special cases...

   Terms are always assumed to be shared and need to be manipulated
   while taking care about references! */

typedef struct eqncell
{
   EqnType        type;      /* Orientation status */
   bool           positive;  /* Equation or inequation? */
   bool           maximal;   /* Is the equation maximal in its clause? */
   bool           was_max;   /* To note changes... */
   bool           equ_lit;   /* Is it a true equational literal or a
				normal literal in equational form? */
   SysDate        ldate;     /* tb_date of terms at freeze date (can */			
   SysDate        rdate;     /* be used to detect rewriting via term */
			     /*	bank. */
   Term_p         lterm;
   Term_p         rterm;
   TB_p           bank;      /* Terms are from this bank */
   struct eqncell *next;     /* For lists of equations */
}EqnCell, *Eqn_p, **EqnRef;

#define EQUAL_PREDICATE "equal"

/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


extern bool EqnUseInfix; /* s = t or EQ(s = t)  ? */
extern bool EqnFullEquationalRep; /* P(x) = $true ? */

#define EqnCellAlloc()    (EqnCell*)SizeMalloc(sizeof(EqnCell))
#define EqnCellFree(junk) SizeFree(junk, sizeof(EqnCell))

Eqn_p   EqnAlloc(Term_p lterm, Term_p rterm, TB_p bank, bool positive);
void    EqnFree(Eqn_p junk);

#define EqnIsOriented(eq) ((eq)->type == EqOriented)

#define EqnIsGround(eq) \
        (TermIsGround((eq)->lterm) && TermIsGround((eq)->rterm))
Eqn_p   EqnParse(Scanner_p in, TB_p bank);
void    EqnPrint(FILE* out, Eqn_p eq, bool negated, bool fullterms);
#define EqnPrintOriginal(out, eq) \
        EqnPrint((out), (eq), normal, true)

void    EqnSwapSidesSimple(Eqn_p eq);
void    EqnSwapSides(Eqn_p eq);
void    EqnRecordTermDates(Eqn_p eq);
EqnSide EqnHasChanged(Eqn_p eq);
bool    EqnSideHasChanged(Eqn_p eq, EqnSide side);
bool    EqnUpdateOrientation(Eqn_p eq);

Eqn_p   EqnCopy(Eqn_p eq, TB_p bank, bool keeporient);

#define EqnIsTrivial(eq, EqualTest) \
        EqualTest((eq)->lterm, (eq)->rterm)

bool    EqnEqualDirected(Eqn_p eq1,  Eqn_p eq2, TermEqualTestFun EqualTest);
bool    EqnEqual(Eqn_p eq1,  Eqn_p eq2, TermEqualTestFun EqualTest);
#define LiteralEqual(eq1, eq2, EqualTest) \
        (((eq1)->positive == (eq2)->positive) \
	 && EqnEqual((eq1),(eq2),(EqualTest)))

bool    EqnSubsumeDirected(Eqn_p subsumer, Eqn_p subsumed, Subst_p
			   subst, TermEqualTestFun EqualTest);
bool    EqnSubsume(Eqn_p subsumer, Eqn_p subsumed, Subst_p subst,
		   TermEqualTestFun EqualTest);
bool    EqnSubsumeP(Eqn_p subsumer, Eqn_p subsumed, 
		    TermEqualTestFun EqualTest);

bool    LiteralSubsumeP(Eqn_p subsumer, Eqn_p subsumed, 
			TermEqualTestFun EqualTest);

#define EqnEquiv(eq1, eq2, EqualTest) \
           (EqnSubsumeP((eq1),(eq2),(EqualTest)) \
	 &&(EqnSubsumeP((eq2),(eq1),(EqualTest))))

#define LiteralEquiv(eq1, eq2, EqualTest) \
        (((eq1)->positive == (eq2)->positive) \
	 && EqnEquiv((eq1),(eq2),(EqualTest)))

bool    EqnUnifyDirected(Eqn_p eq1, Eqn_p eq2, Subst_p subst);
bool    EqnUnify(Eqn_p eq1, Eqn_p eq2, Subst_p subst);
bool    EqnUnifyP(Eqn_p eq1, Eqn_p eq2);

bool          EqnOrient(OCB_p ocb, Eqn_p eq);

CompareResult EqnCompare(OCB_p ocb, Eqn_p eq1, Eqn_p eq2);
bool          EqnGreater(OCB_p ocb, Eqn_p eq1, Eqn_p eq2);
CompareResult LiteralCompare(OCB_p ocb, Eqn_p eq1, Eqn_p eq2);
bool          LiteralGreater(OCB_p ocb, Eqn_p eq1, Eqn_p eq2);

FunCode SubstNormEqn(Eqn_p eq, Subst_p subst, VarBank_p vars);

double  EqnWeight(Eqn_p eq, double max_multiplier, long vweight, long
		  fweight);
double  LiteralWeight(Eqn_p eq, double max_multiplier, double
		      pos_multiplier, long vweight, long fweight);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





