/*-----------------------------------------------------------------------

File  : ccl_eqnlist.h

Author: Stephan Schulz

Contents
 
   Functions for dealing with (singly linked) lists of equations as
   used in clauses.  

Changes

<1> Fri Apr 10 16:46:17 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CCL_EQNLIST

#define CCL_EQNLIST

#include <ccl_eqn.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/




/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

void    EqnListFree(Eqn_p list);

long    EqnListLength(Eqn_p list);

Eqn_p   EqnListExtractElement(EqnRef element);
#define EqnListExtractFirst(list)\
        EqnListExtractElement(list)
void    EqnListDeleteElement(EqnRef element);
#define EqnListDeleteFirst(list)\
        EqnListDeleteElement(list)
void    EqnListInsertElement(EqnRef pos, Eqn_p element);
#define EqnListInsertFirst(list, element)\
        EqnListDeleteElement((list), (element))
Eqn_p   EqnListAppend(EqnRef list, Eqn_p newpart);
Eqn_p   EqnListCopy(Eqn_p list, TB_p bank, bool keeporient);
Eqn_p   EqnListCopyExcept(Eqn_p list, Eqn_p except, TB_p bank, bool
			  keeporient);
Eqn_p   EqnListNegateEqns(Eqn_p list);
int     EqnListRemoveDuplicates(Eqn_p list, TermEqualTestFun
				EqualTest);
int     EqnListRemoveResolved(EqnRef list, TermEqualTestFun
			      EqualTest);
bool    EqnListIsTautologic(Eqn_p list);
bool    EqnListIsGround(Eqn_p list);
void    EqnListResetOrientStatus(Eqn_p list, EqnType status);
long    EqnListOrient(OCB_p ocb, Eqn_p list);
long    EqnListUpdateOrientation(Eqn_p list);
long    EqnListMaximalLiterals(OCB_p ocb, Eqn_p list);
bool    EqnListEqnIsMaximal(OCB_p ocb, Eqn_p list, Eqn_p eqn);
void    EqnListMarkRestrictedTerms(Eqn_p list);

void    EqnListPrint(FILE* out, Eqn_p list, char* sep, 
		     bool negated,  bool fullterms);
Eqn_p   EqnListParse(Scanner_p in, TB_p bank, TokenType sep);

FunCode NormSubstEqnListExcept(Eqn_p list, Eqn_p except, Subst_p
			       subst, VarBank_p vars);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





