/*-----------------------------------------------------------------------

File  : ccl_pdtrees.h

Author: Stephan Schulz

Contents
 
  Perfect discrimination trees for optimized rewriting.

Changes

<1> Mon Jun 22 17:04:32 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CCL_PDTREES

#define CCL_PDTREES

#include <clb_pdarrays.h>
#include <ccl_clausepos.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

#define PDT_INTERFACE_WATCH 512


/* A cell in the perfect discrimination tree... */

typedef struct pdtreecell
{
   struct pdtreecell  **f_alternatives; /* function alternatives */
   PDArray_p          v_alternatives;   /* Variables */
   FunCode            max_var;          /* Largest variable index
					   (negated f_code) occuring
					   at this node */
   Term_p             variable;         /* If variable, which one? */
   struct pdtreecell  *parent;          /* Where did we come from? */
   FunCode            sig_size;         /* Number of function symbol
					   alternatives */
   long               ref_count;        /* How many terms share this
					   node? */
   PTree_p            demod_terms;      /* Denoted by ClausePosCells */
   FunCode            trav_state;       /* For traversing during
					   matching */
   PStackPointer      var_state;        /* For storing information
					   about variables bindings */
}PDTreeCell, *PDTree_p;

/* A PDSearch state, just to keep all things in one place */

typedef struct pdsearchcell
{
   PDTree_p trav_pointer;
   PStack_p term_state;
   PStack_p term_proc;
   PStack_p demod_store;
}PDSearchCell, *PDSearch_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

extern bool PDTreePreferGeneral;

#define  PDTreeCellAlloc()    (PDTreeCell*)SizeMalloc(sizeof(PDTreeCell))
#define  PDTreeCellFree(junk) SizeFree(junk, sizeof(PDTreeCell))
PDTree_p PDTreeAlloc(FunCode sig_size);
void     PDTreeDefaultCellFree(PDTree_p root);
void     PDTreeFree(PDTree_p root);

#define    PDSearchCellAlloc() (PDSearchCell*)SizeMalloc(sizeof(PDSearchCell))
#define    PDSearchCellFree(junk) SizeFree(junk, sizeof(PDSearchCell))
PDSearch_p PDSearchAlloc(PDTree_p index, Term_p term);
void       PDSearchFree(PDSearch_p junk);

PStack_p TermLRTraverseInit(Term_p term);
Term_p   TermLRTraverseNext(PStack_p stack);
Term_p   TermLRTraversePrev(PStack_p stack, Term_p term);
#define  TermLRTraverseExit(stack) PStackFree(stack)

void     PDTreeInsert(PDTree_p root, ClausePos_p demod_side);
void     PDTreeDelete(PDTree_p root, Term_p term, Clause_p restrict);

PDTree_p PDTreeFindNextIndexedTerm(PDSearch_p state, Subst_p subst);

ClausePos_p PDTreeFindNextDemodulator(PDSearch_p state, Subst_p
				      subst);

PDTree_p PDTreeFindTerm(PDTree_p index, Term_p term);
void PDTreeStoreClauses(PTree_p *tree, PDTree_p index, Term_p term);

bool ClauseHasIndexedTerm(Clause_p clause);
long ClauseRemoveUnchangedIndexedTerms(Clause_p clause, PDTree_p
				       index);
long ClauseRemoveFromIndex(Clause_p clause, PDTree_p index);

#endif 

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





