/*-----------------------------------------------------------------------

File  : cco_eqnresolving.c

Author: Stephan Schulz

Contents
 
  Functions controlling equality resolution.

Changes

<1> Mon Jun  8 17:17:57 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "cco_eqnresolving.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: ComputeAllEqnResolvents()
//
//   Compute all equality resolvents of clause, make parent_alias
//   their only parent and put them into store.
//
// Global Variables: -
//
// Side Effects    : As above
//
/----------------------------------------------------------------------*/

long ComputeAllEqnResolvents(TB_p bank, Clause_p clause, Clause_p
			     parent_alias, ClauseSet_p store,
			     VarBank_p freshvars)
{
   Clause_p    resolvent;
   Eqn_p       test;
   ClausePos_p pos;
   long        resolv_count = 0;
   
   pos = ClausePosAlloc();

   test = ClausePosFirstEqResLiteral(clause, pos);
   
   while(test)
   {
      resolvent = ComputeEqRes(bank, pos, freshvars);
      if(resolvent)
      {
	 resolv_count++;
	 resolvent->parent1 = parent_alias;
	 ClauseRegisterChild(parent_alias, resolvent);
	 ClauseSetInsert(store, resolvent);
      }
      test = ClausePosNextEqResLiteral(pos);
   }
   ClausePosFree(pos);

   return resolv_count;
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


