/*-----------------------------------------------------------------------

File  : cco_factoring.c

Author: Stephan Schulz

Contents
 
  Functions controlling factoring.

Changes

<1> Mon Jun  8 17:17:57 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "cco_factoring.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: ComputeAllOrderedFactors()
//
//   Compute all ordered factors of clause, make parent_alias their
//   only parent and put them into store. Return number of factors.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

long ComputeAllOrderedFactors(TB_p bank, OCB_p ocb,
			      Clause_p clause, Clause_p parent_alias,
			      ClauseSet_p store, VarBank_p freshvars)
{
   Clause_p    factor;
   Eqn_p       test;
   ClausePos_p pos1, pos2;
   long        factor_count = 0;

   pos1 = ClausePosAlloc();
   pos2 = ClausePosAlloc();

   test = ClausePosFirstOrderedFactorLiterals(clause, pos1, pos2);
   
   while(test)
   {
      factor = ComputeOrderedFactor(bank, ocb, pos1, pos2, freshvars);
      if(factor)
      {
	 factor_count++;
	 factor->parent1 = parent_alias;
	 ClauseRegisterChild(parent_alias, factor);
	 ClauseSetInsert(store, factor);
      }
      test = ClausePosNextOrderedFactorLiterals(pos1, pos2);
   }
   ClausePosFree(pos1);
   ClausePosFree(pos2); 
   return factor_count++;
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


