/*-----------------------------------------------------------------------

File  : che_clauseweight.h

Author: Stephan Schulz

Contents
 
  Evaluation of a clause by clause weight, also an example for setting
  up an evaluation function.

Changes

<1> Sat Jul  5 02:28:25 MET DST 1997
    New

-----------------------------------------------------------------------*/

#ifndef CHE_CLAUSEWEIGHT

#define CHE_CLAUSEWEIGHT

#include <che_wfcb.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

#define DEFAULT_MAX_MULT 1
#define DEFAULT_POS_MULT 1
#define DEFAULT_VWEIGHT  1
#define DEFAULT_FWEIGHT  1

typedef struct weightparamcell
{
   double max_multiplier;
   double pos_multiplier;
   long   vweight;
   long   fweight;
}WeightParamCell, *WeightParam_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


#define WeightParamCellAlloc() (WeightParamCell*) \
        SizeMalloc(sizeof(WeightParamCell))
#define WeightParamCellFree(junk) \
        SizeFree(junk, sizeof(WeightParamCell))
   
WFCB_p ClauseWeightInit(WFCBPARAMETERS);
WFCB_p ClauseGoalWeightInit(WFCBPARAMETERS);
WFCB_p ClauseAxiomWeightInit(WFCBPARAMETERS);

Eval_p ClauseWeightCompute(void* data, Clause_p clause);
Eval_p ClauseGoalWeightCompute(void* data, Clause_p clause);
Eval_p ClauseAxiomWeightCompute(void* data, Clause_p clause);

void   ClauseWeightExit(void* data);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





