/*-----------------------------------------------------------------------

File  : che_fifo.c

Author: Stephan Schulz

Contents
 
  FIFO-Clause evaluation

Changes

<1> Fri Jun  5 22:51:52 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "che_fifo.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


   

/*-----------------------------------------------------------------------
//
// Function: FIFOEvalInit()
//
//   Return an initialized WFCB for FIFO evaluation.
//
// Global Variables: 
//
// Side Effects    : 
//
/----------------------------------------------------------------------*/

WFCB_p FIFOEvalInit(WFCBPARAMETERS)
{
   double *data;

   data = SizeMalloc(sizeof(double));
   *data = 0.0;

   return WFCBAlloc(FIFOEvalCompute, FIFOEvalExit, data);
}


/*-----------------------------------------------------------------------
//
// Function: FIFOEvalCompute()
//
//   Compute an evaluation for a clause.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

Eval_p FIFOEvalCompute(void* data, Clause_p clause)
{
   Eval_p eval   = EvalAlloc();
   double *local = data;
   
   eval->priority  = PrioNormal;
   eval->heuristic = *local;
   *local = (*local)+1.0;

   return eval;
}


/*-----------------------------------------------------------------------
//
// Function: FIFOEvalExit()
//
//   Free the data entry in a FIFO WFCB.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void FIFOEvalExit(void* data)
{
   SizeFree(data, sizeof(double));
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


