/*-----------------------------------------------------------------------

File  : che_heu_standard.c

Author: Stephan Schulz

Contents
 
  

Changes

<1> Mon Jun  8 02:32:47 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "che_heu_standard.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: HCBStandardCreate()
//
//   Create a HCB entry for the standard heuristic. The parameter
//   stacks are interpreted as follows:
// 
//   Stack        What?               Default
//                                    
//   i_parms:  0  ClauseWeight-Steps  DEFAULT_CLAUSEWEIGHT_STEPS
//             1  FIFO-Steps          DEFAULT_FIFO_STEPS
//             2  VWeigth             DEFAULT_VWEIGHT
//             3  FWeight             DEFAULT_FWEIGHT
//   d_parms:  0  Max-Multiplier      DEFAULT_MAX_MULT
//             1  Pos-Multiplier      DEFAULT_POS_MULT
//   s_parms:  Ignored
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

HCB_p HCBStandardCreate(WFCBPARAMETERS)
{
   long          fweight, vweight, wsteps, fifosteps;
   double        max_mult, pos_mult;
   PStackPointer psp;
   DStackPointer dsp;
   PStack_p      i_stack = PStackAlloc();
   DStack_p      d_stack = DStackAlloc();
   WFCB_p        wfcb;
   HCB_p         hcb     = HCBCellAlloc();
   

   psp = PStackGetSP(i_parms);
   dsp = DStackGetSP(d_parms);
   
   if(psp > 4)
   {
      Error("Heuristic \"standard\" takes at most 4 integer "
	    "parameters", USAGE_ERROR);
   }      
   fweight   = psp >= 4 ? 
      PStackElementInt(i_parms, 3) : DEFAULT_FWEIGHT;
   vweight   = psp >= 3 ? 
      PStackElementInt(i_parms, 2) : DEFAULT_VWEIGHT;
   fifosteps = psp >= 2 ?
      PStackElementInt(i_parms, 1) : DEFAULT_FIFO_STEPS;
   wsteps    = psp >= 1 ? 
      PStackElementInt(i_parms, 0) : DEFAULT_CLAUSEWEIGHT_STEPS;

   if(dsp > 2)
   {
      Error("Heuristic \"standard\" takes at most 2 float "
	    "parameters", USAGE_ERROR);
   }      
   max_mult = dsp >= 2 ? 
      DStackElement(d_parms, 1) : DEFAULT_MAX_MULT;
   pos_mult = dsp >= 1 ?
      DStackElement(d_parms, 0) : DEFAULT_POS_MULT;
   
   PStackPushInt(i_stack, vweight);
   PStackPushInt(i_stack, fweight);
   DStackPush(d_stack, max_mult);
   DStackPush(d_stack, pos_mult);
      
   
   wfcb = ClauseWeightInit(NULL, NULL, NULL, i_stack, d_stack, NULL);
   hcb->wfcb_list = wfcb;
   assert(wfcb->next == NULL);
   
   wfcb = FIFOEvalInit(NULL, NULL, NULL, NULL, NULL, NULL);
   wfcb->next = hcb->wfcb_list;
   hcb->wfcb_list = wfcb;
   
   hcb->wfcb_no          = 2;
   hcb->current_eval     = 0;
   hcb->select_switch[0] = wsteps;
   hcb->select_switch[1] = wsteps+fifosteps;
   hcb->select_count     = 0;

   hcb->hcb_select       = HCBStandardClauseSelect;
   hcb->hcb_exit         = NULL;
   hcb->data             = NULL;

   PStackFree(i_stack);
   DStackFree(d_stack);
   return hcb;
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


