/*-----------------------------------------------------------------------

File  : che_heuristics.c

Author: Stephan Schulz

Contents
 
  High-Level interface functions to the heuristics module.


Changes

<1> Mon Jun  8 02:14:51 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "che_heuristics.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

HeuristicAssocCell HeuristicsTable[]=
{
   {HEU_STANDARD,     "Standard",      HCBStandardCreate},
   {HEU_WEIGHT,       "Clauseweight",  HCBClauseweightCreate},
   {HEU_REFINED,      "Refinedweight", HCBRefinedWeightCreate},
   {HEU_FIFO,         "FIFOWeight",    HCBFIFOCreate},
   {HEU_EX_STANDARD,  "ExStandard",    HCBExStandardCreate},
   {HEU_NO_HEURISTIC, NULL,       NULL}
};

/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: HCBCreate() 
//
//   Given a name and the necessary parameters, return a HCB.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

HCB_p HCBCreate(char* name, WFCBPARAMETERS)
{
   int    i;
   DStr_p err;

   for(i=0; HeuristicsTable[i].heuristic; i++)
   {
      if(strcmp(HeuristicsTable[i].name, name)==0)
      {
	 return HeuristicsTable[i].heuristic_create(ocb, sig,
						    clauses,
						    i_parms,
						    d_parms,
						    s_parms);
      }
   }
   err = DStrAlloc();
   DStrAppendStr(err, "Heuristic \"");
   DStrAppendStr(err, name);
   DStrAppendStr(err, " unknown \"");
   Error(DStrView(err), USAGE_ERROR);
   DStrFree(err);
   
   return NULL;
}



/*-----------------------------------------------------------------------
//
// Function: HeuristicParmsAlloc()
//
//   Allocate a cell for parameters, with initialized empty stacks.
//
// Global Variables: -
//
// Side Effects    : Memory Operations
//
/----------------------------------------------------------------------*/

HeuristicParms_p HeuristicParmsAlloc()
{
   HeuristicParms_p handle = HeuristicParmsCellAlloc();
   
   handle->heuristic_name = "standard";
   handle->i_parms = PStackAlloc();
   handle->d_parms = DStackAlloc();
   handle->s_parms = PStackAlloc();

   return handle;
}



/*-----------------------------------------------------------------------
//
// Function: HeuristicParmsFree()
//
//   Free a parameter cell.
//
// Global Variables: -
//
// Side Effects    : Memory Operations
//
/----------------------------------------------------------------------*/

void HeuristicParmsFree(HeuristicParms_p junk)
{
   assert(junk);
   
   junk->heuristic_name = NULL;
   PStackFree(junk->i_parms);
   DStackFree(junk->d_parms);
   PStackFree(junk->s_parms);

   HeuristicParmsCellFree(junk);
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


