/*-----------------------------------------------------------------------

File  : che_heuristics.h

Author: Stephan Schulz

Contents
 
  High-Level interface functions to the heuristics module.

Changes

<1> Mon Jun  8 02:14:51 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CHE_HEURISTICS

#define CHE_HEURISTICS


#include <che_hcb.h>
#include <che_heu_standard.h>
#include <che_heu_exstandard.h>
#include <che_heu_weight.h>
#include <che_heu_refinedweight.h>
#include <che_heu_fifo.h>
#include <che_heu_lifo.h>


/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef enum
{
   HEU_NO_HEURISTIC = 0,
   HEU_STANDARD,
   HEU_WEIGHT,
   HEU_REFINED,
   HEU_FIFO,
   HEU_EX_STANDARD
}Heuristic;

typedef struct heuristic_assoc_cell
{
   Heuristic    heuristic;
   char*        name;
   HCBCreateFun heuristic_create;
}HeuristicAssocCell, HeuristicAssoc_p;


typedef struct heuristic_parms_cell
{
   char     *heuristic_name;
   PStack_p i_parms;
   DStack_p d_parms;
   PStack_p s_parms;
}HeuristicParmsCell, *HeuristicParms_p;

/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


extern  HeuristicAssocCell HeuristicsTable[];

HCB_p   HCBCreate(char* name, WFCBPARAMETERS);

#define HeuristicParmsCellAlloc() \
   (HeuristicParmsCell*)SizeMalloc(sizeof(HeuristicParmsCell))
#define HeuristicParmsCellFree(junk) \
   SizeFree(junk, sizeof(HeuristicParmsCell))

HeuristicParms_p HeuristicParmsAlloc();
void             HeuristicParmsFree(HeuristicParms_p junk);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





