/*-----------------------------------------------------------------------

File  : che_lifo.c

Author: Stephan Schulz

Contents
 
  LIFO-Clause evaluation

Changes

<1>Mon Jun 22 15:28:23 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "che_lifo.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


   

/*-----------------------------------------------------------------------
//
// Function: LIFOEvalInit()
//
//   Return an initialized WFCB for FIFO evaluation.
//
// Global Variables: 
//
// Side Effects    : 
//
/----------------------------------------------------------------------*/

WFCB_p LIFOEvalInit(WFCBPARAMETERS)
{
   double *data;

   data = SizeMalloc(sizeof(double));
   *data = 0.0;

   return WFCBAlloc(LIFOEvalCompute, LIFOEvalExit, data);
}


/*-----------------------------------------------------------------------
//
// Function: LIFOEvalCompute()
//
//   Compute an evaluation for a clause.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

Eval_p LIFOEvalCompute(void* data, Clause_p clause)
{
   Eval_p eval   = EvalAlloc();
   double *local = data;
   
   eval->priority  = PrioNormal;
   eval->heuristic = *local;
   *local = (*local)-1.0;

   return eval;
}


/*-----------------------------------------------------------------------
//
// Function: LIFOEvalExit()
//
//   Free the data entry in a LIFO WFCB.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void LIFOEvalExit(void* data)
{
   SizeFree(data, sizeof(double));
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


