/*-----------------------------------------------------------------------

File  : che_refinedweight.h

Author: Stephan Schulz

Contents
 
  Evaluation of a clause by refined clause weight (using product of
  term weights in an equation as well as sum.

Changes

<1> Wed Jun 17 00:11:03 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CHE_REFINEDWEIGHT

#define CHE_REFINEDWEIGHT

#include <che_clauseweight.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

#define DEFAULT_PRODUCT_INFLUENCE 0.2

typedef struct refinedweightparamcell
{
   double max_multiplier;
   double pos_multiplier;
   double product_influence;
   long   vweight;
   long   fweight;
}RefinedWeightParamCell, *RefinedWeightParam_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


#define RefinedWeightParamCellAlloc() (RefinedWeightParamCell*) \
        SizeMalloc(sizeof(RefinedWeightParamCell))
#define RefinedWeightParamCellFree(junk) \
        SizeFree(junk, sizeof(RefinedWeightParamCell))
   
WFCB_p ClauseRefinedWeightInit(WFCBPARAMETERS);

Eval_p ClauseRefinedWeightCompute(void* data, Clause_p clause);

void   ClauseRefinedWeightExit(void* data);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





