/*-----------------------------------------------------------------------

File  : che_simweight.h

Author: Stephan Schulz

Contents
 
  Evaluation of a clause by similarity of terms (equations with
  similar terms have low weight).

Changes

<1> Sun Jun 28 18:18:00 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CHE_SIMWEIGHT

#define CHE_SIMWEIGHT

#include <che_wfcb.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

#define DEFAULT_EQUAL_WEIGHT     2.0
#define DEFAULT_VAR_VAR_CLASH    1.0
#define DEFAULT_VAR_TERM_CLASH   2.0
#define DEFAULT_TERM_TERM_CLASH  3.0

typedef struct simparamcell
{
   double equal_weight;
   double var_var_clash;
   double var_term_clash;
   double term_term_clash;
}SimParamCell, *SimParam_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


#define SimParamCellAlloc() (SimParamCell*) \
        SizeMalloc(sizeof(SimParamCell))
#define SimParamCellFree(junk) \
        SizeFree(junk, sizeof(SimParamCell))
   
WFCB_p SimWeightInit(WFCBPARAMETERS);

Eval_p SimWeightCompute(void* data, Clause_p clause);

void   SimWeightExit(void* data);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





