/*-----------------------------------------------------------------------

File  : cto_basefuns.c

Author: Stephan Schulz

Contents
 
  Some elementary functions for ordering-related stuff that did not
  really belong anywhere else...

Changes

<1> Mon Jun 22 17:51:53 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "cto_basefuns.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: TermSetVarCounter()
//
//   Set the varcounter of variable cells in term according to the
//   following table, ignoring bindindings...
//
//   term->var_freqency  code    result
//
//   *                   NoSide    NoSide
//   NoSide              val       val
//   LeftSide            LeftSide  LeftSide
//                       NoSide    LeftSide
//                       RightSide BothSides
//   RightSide           RightSide RightSide
//                       NoSide    RightSide
//                       LeftSide  BothSides
//   BothSides           R/L-Side  BothSides
//    
//   Otherwise:          assertion error...                 
//
// Global Variables: -
//
// Side Effects    : Resets TPOpFlag of term nodes
//
/----------------------------------------------------------------------*/

void TermSetVarCounter(Term_p term, EqnSide code)
{
   PStack_p stack= PStackAlloc();
   int      i;
   
   PStackPushP(stack, term);

   while(!PStackEmpty(stack))
   {
      term = PStackPopP(stack);
      TermCellDelProp(term, TPOpFlag);

      if(TermIsVar(term))
      {
	 switch(code)
	 {
	 case NoSide:
	       term->var_frequency = NoSide;
	       break;
	 case LeftSide:
	       if(term->var_frequency==RightSide ||
		  term->var_frequency==BothSides)		  
	       {
		  term->var_frequency = BothSides;
	       }
	       else
	       {
		  term->var_frequency = LeftSide;
	       }
	       break;
	 case RightSide:
	       if(term->var_frequency==LeftSide ||
		  term->var_frequency==BothSides)		  
	       {
		  term->var_frequency = BothSides;
	       }
	       else
	       {
		  term->var_frequency = RightSide;
	       }
	       break;
	 default:
	       assert(false);
	       break;
	 }
      }
      else
      {
	 for(i=0; i<term->arity; i++)
	 {
	    PStackPushP(stack, term->args[i]);
	 }
      }   
   }             
}


/*-----------------------------------------------------------------------
//
// Function: EqnSetVarCounter()
//
//   Set the varcounter for terms in eqn so it can be recognized in
//   which terms variables appear.
//
// Global Variables: -
//
// Side Effects    : Changes varfrequency, TPOpFlag of terms in eqn.
//
/----------------------------------------------------------------------*/

void TermPairSetVarCounter(Term_p lterm, Term_p rterm)
{
   TermSetVarCounter(lterm, NoSide);
   TermSetVarCounter(rterm, NoSide);
   TermSetVarCounter(lterm, LeftSide);
   TermSetVarCounter(rterm, RightSide);
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


