/*-----------------------------------------------------------------------

File  : cto_ocb.h

Author: Stephan Schulz

Contents
 
  Global definitions for orderings: Comparison results, precedences,
  order control blocks.

Changes

<1> Wed Apr 29 02:51:28 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CTO_OCB

#define CTO_OCB

#include <cte_termbanks.h>


/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef enum
{
   to_uncomparable,
   to_equal,
   to_greater,
   to_lesser
}CompareResult;


typedef enum 
{
   NoOrdering,
   KBO,
   KBO2,
   LPO,
   RPO
}TermOrdering;

typedef enum
{
   Multiset,
   LeftRight,
   RightLeft
}CompareStatus;



typedef struct ocb_cell
{
   TermOrdering   type;
   Sig_p          sig;         /* Slightly hacked...this is only an
				  unsupervised reference ! */
   int            sig_size;
   long          *weights;     /* Array of weights */
   CompareStatus *status;      /* Array of status  */
   CompareResult *precedence;  /* Interpreted as two-dimensional
				   array */
   PStack_p       statestack;  /* Contains backtrack information */
}OCBCell, *OCB_p;



/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define OCBCellAlloc() (OCBCell*)SizeMalloc(sizeof(OCBCell))
#define OCBCellFree(junk)        SizeFree(junk, sizeof(OCBCell))

/* Symbolic representation of ordering relations */

extern char*  TOCompareSymbol[];

OCB_p         OCBAlloc(TermOrdering type, Sig_p sig);
void          OCBFree(OCB_p junk);

void          OCBDebugPrint(FILE* out, OCB_p ocb);

CompareResult OCBPrecedenceInverse(CompareResult relation);
PStackPointer OCBPrecedenceAddTuple(OCB_p ocb, FunCode f1, FunCode f2,
				    CompareResult relation);

bool          OCBPrecedenceBacktrack(OCB_p ocb, PStackPointer state);
#define OCBPrecedenceGetState(ocb) \
              PStackGetSP((ocb)->statestack)

/* Getting the addresses of OCB entries for modification */

#define OCBFunWeightPos(ocb, f) \
              (assert((f)>0), assert((f)<=(ocb)->sig_size), &((ocb)->weights[(f)-1]))
#define OCBFunComparePos(ocb, f1, f2) \
              (assert((f1)>0), assert((f2)>0), assert((f1)<=(ocb)->sig_size),\
	       assert((f2)<=(ocb)->sig_size),\
              (&((ocb)->precedence[((f2)-1)*(ocb)->sig_size+((f1)-1)])))
#define OCBFunStatusPos(ocb, f) \
              (&((ocb)->status[(f)-1]))

/* Functions for Querying the OCB */

#define OCBFunWeight(ocb,f) (*(OCBFunWeightPos((ocb), (f))))
#define OCBFunCompare(ocb, f1, f2) \
              (*OCBFunComparePos((ocb), (f1), (f2)))
#define OCBFunStatus(ocb,f) (*(OCBFunStatusPos((ocb), (f))))

FunCode       OCBTermMaxFunCode(OCB_p ocb, Term_p term);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





