/*-----------------------------------------------------------------------

File  : cte_varmanip.c

Author: Stephan Schulz

Contents
 
  Functions for normalizing variable names in terms.

Changes

<1> Mon Feb 16 12:25:42 MET 1998

-----------------------------------------------------------------------*/

#include "cte_varmanip.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: VarManipAlloc()
//
//   Allocate an initialized VarManipCell.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

VarManip_p VarManipAlloc()
{
   VarManip_p handle;

   handle = VarManipCellAlloc();   
   handle->var_counter = 0;
   handle->var_map = NULL;
   
   return handle;
}


/*-----------------------------------------------------------------------
//
// Function: VarManipFree()
//
//   Free a VarManip data structure
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void VarManipFree(VarManip_p junk)
{
   NumTreeFree(junk->var_map);
   VarManipCellFree(junk);
}


/*-----------------------------------------------------------------------
//
// Function:  TermComputeNormVars()
//
//   Compute a numtree mapping the variables in term to norm variables
//   (x1,...,xn, internally encoded by long ints < 0). Uses existing
//   bindings in newvars, can be called repeatedly to norm ordered
//   list of terms. Returns number of newly bound variables.
//
// Global Variables: -
//
// Side Effects    : Changes the NumTree at newvars, memory operations
//
/----------------------------------------------------------------------*/

long TermComputeNormVars(Term_p term, VarManip_p newvars)
{
   PStack_p stack = PStackAlloc();
   int i;
   long count = 0;
   IntOrP arg;
   
   PStackPushP(stack, term);

   while(!PStackEmpty(stack))
   {
      term = PStackPopP(stack);
      if(TermIsVar(term))
      {
	 arg.i_val = (newvars->var_counter-1); /* Next variable code
						  to use... */
	 if(NumTreeStore(&(newvars->var_map),  term->f_code, arg, arg))
	 { /* This is a new binding */
	    newvars->var_counter--;
	    count++;
	 }
      }
      else
      {
	 for(i=term->arity-1; i>=0; i--)
	 {
	    PStackPushP(stack, term->args[i]);
	 }
      }
   }
   PStackFree(stack);
   return count;
}



Term_p TermApplyVarRenaming(Term_p term,  VarManip_p newvars);


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





