/*-----------------------------------------------------------------------

File  : cte_vartrans.c

Author: Stephan Schulz

Contents
 
  Funktions for the translation of external print representations and
  internal f_codes for variables. 

Changes

<1> Mon Nov 10 00:00:16 MET 1997
    New

-----------------------------------------------------------------------*/

#include <cte_vartrans.h>



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function:  VarTransAlloc()
//
//   Allocate an empty, initialized VarTrans-Structure, return pointer
//   to it.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

VarTrans_p VarTransAlloc()
{
   VarTrans_p handle;

   handle = VarTransCellAlloc();
   handle->v_count = 0;
   handle->v_index = NULL;

   return handle;
}


/*-----------------------------------------------------------------------
//
// Function: VarTransFree()
//
//   Deallocate a VarTransCell.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void VarTransFree(VarTrans_p junk)
{
   assert(junk);
   StrTreeFree(junk->v_index);
   VarTransCellFree(junk);
}

/*-----------------------------------------------------------------------
//
// Function: VarTransClear()
//
//   Reset the status of a VarTrans Structure, delete all
//   associations. 
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void VarTransClear(VarTrans_p vars)
{
   StrTreeFree(vars->v_index);
   vars->v_index = NULL;
   vars->v_count = 0;
}


/*-----------------------------------------------------------------------
//
// Function: VarTransFindFCode()
//
//   Return the f_code for a variable if it is known, 0 otherwise. 
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

FunCode VarTransFindFCode(VarTrans_p vars, char* name)
{
   StrTree_p entry;
   
   entry = StrTreeFind(vars->v_index, name);
   
   if(entry)
   {
      return entry->val1.i_val;
   }
   return 0;
}

/*-----------------------------------------------------------------------
//
// Function: VarTransInsertId()
//
//   Return a numerical code for a string variable and store the
//   association in vars if it is new.
//
// Global Variables: -
//
// Side Effects    : Changes state of *vars
//
/----------------------------------------------------------------------*/

FunCode VarTransInsertId(VarTrans_p vars, char* name)
{
   StrTree_p handle, test;

   handle = StrTreeFind(vars->v_index, name);

   if(!handle)
   {
      handle = StrTreeCellAlloc();
      handle->key = SecureStrdup(name);
      handle->val1.i_val = -(++(vars->v_count));
      test = StrTreeInsert(&(vars->v_index), handle);
      assert(test == NULL);
   }
   return handle->val1.i_val;
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


