/*-----------------------------------------------------------------------

File  : cte_vartrans.h

Author: Stephan Schulz

Contents
 
  Funktions for the translation of external print representations and
  internal f_codes for variables, also for normalizing variables and
  for handing out fresh variables.

Changes

<1> Sat Jul  5 02:28:25 MET DST 1997
    New

-----------------------------------------------------------------------*/

#ifndef CTE_VARTRANS

#define CTE_VARTRANS

#include "cte_types.h"
#include <clb_stringtrees.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

/* Signature-Equivalent for variables: Associate identifier with
   internal encoding. I don't think we need an efficient association
   for the opposite direction - if I'm wrong I'll add it later. */

typedef struct vartranscell
{
   long      v_count;
   StrTree_p v_index;
}VarTransCell, *VarTrans_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


#define VarTransCellAlloc() (VarTransCell*)SizeMalloc(sizeof(VarTransCell))
#define VarTransCellFree(junk)         SizeFree(junk, sizeof(VarTransCell))

VarTrans_p VarTransAlloc();
void       VarTransFree(VarTrans_p junk);
void       VarTransClear(VarTrans_p vars);

FunCode    VarTransFindFCode(VarTrans_p vars, char* name);
FunCode    VarTransInsertId(VarTrans_p vars, char* name);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





