/*-----------------------------------------------------------------------

File  : cte_termbanks.h

Author: Stephan Schulz

Contents
 
  Definitions for term banks - i.e. shared representations of terms as
  defined in cte_terms.h. Uses the same struct, but adds
  administrative stuff and functionality for sharing.

  There are two sets of funktions for the manangment of term trees:
  Funktions operating only on the top cell, and functions descending
  the term structure. Top level functions implement a conventional AVL
  tree with key f_code.masked_properties.entry_nos_of_args and are
  implemented in cte_termtrees.[ch]

Changes

<1> Mon Sep 22 00:15:39 MET DST 1997
    New
<2> Wed Feb 25 18:16:34 MET 1998
    Adapted for use of new term modules with shared variables


-----------------------------------------------------------------------*/

#ifndef CTE_TERMBANKS

#define CTE_TERMBANKS

#include <cio_basicparser.h>
#include <cte_termtrees.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef struct tbcell
{
   TermProperties prop_mask;  /* Which properties are used in this
				 term bank? */ 
   unsigned long in_count;    /* How many terms have been inserted? */
   NumTree_p     entry_index; /* Associate entry_no with address -
				 necessary for ordered output */
   NumTree_p     ext_index;   /* Associate _external_ abbreviations (=
				 entry_no's with term nodes, necessary
				 for parsing of term bank terms */
   Sig_p         sig;         /* Store sig info */
   VarBank_p     vars;        /* Information about (shared) variables */
   Term_p        true_term;   /* Pointer to the special term with the
				 $true constant. */
   SysDate       tb_date;     /* Is incremented whenever a new term is
				 inserted into the term tree. */
   Term_p        term_root;   /* Here are the terms */
}TBCell, *TB_p;
   


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

extern bool TBPrintTermsFlat;
extern bool TBPrintInternalInfo;
extern bool TBSupportReplace;

#define TBCellAlloc() (TBCell*)SizeMalloc(sizeof(TBCell))
#define TBCellFree(junk)         SizeFree(junk, sizeof(TBCell))

TB_p    TBAlloc(TermProperties prop_mask, Sig_p sig);
void    TBFree(TB_p junk);

Term_p  DefaultSharedTermCellAlloc();
void    TermGetRef(Term_p *ref, Term_p term);
void    TermReleaseRef(Term_p *ref);
void    TermIntroduceSuperterm(Term_p term, Term_p superterm);
bool    TermRemoveSuperterm(Term_p term, Term_p superterm);

bool    TBTermEqual(Term_p t1, Term_p t2);
bool    TBTermStructEqual(Term_p t1, Term_p t2);
#define TBTermIsSubterm(super, term)\
        TermIsSubterm((super),(term),DEREF_NEVER,TBTermEqual)
Term_p  TBInsert(TB_p bank, Term_p term, DerefType deref);
Term_p  TBFind(TB_p bank, Term_p term);
bool    TBDelete(TB_p bank, Term_p term);
void    TBPrintBankInOrder(FILE* out, TB_p bank);
void    TBPrintTermCompact(FILE* out, TB_p bank, Term_p term);
#define TBPrintTermFull(out, bank, term) \
        TermPrint((out), (term), (bank)->sig, DEREF_NEVER)
void    TBPrintTerm(FILE* out, TB_p bank, Term_p term, bool fullterms);
void    TBPrintBankTerms(FILE* out, TB_p bank);
Term_p  TBTermParse(Scanner_p in, TB_p bank);
int     TBTermParseArgList(Scanner_p in, Term_p** arg_anchor,
			  TB_p bank);
long    TBCountSupertermReferences(Term_p term);
long    TBCountTotalReferences(Term_p term);
bool    TBVerifyReferences(FILE* out, Term_p term);
Term_p  TBVerifyBankReferences(FILE* out, TB_p bank);

void    TBRefSetProp(TB_p bank, TermRef ref, TermProperties prop);
void    TBRefDelProp(TB_p bank, TermRef ref, TermProperties prop);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





