/*-----------------------------------------------------------------------

File  : cte_termpos.c

Author: Stephan Schulz

Contents
 
  Positions in terms

Changes

<1> Sun May 10 17:40:18 MET DST 1998

-----------------------------------------------------------------------*/

#include "cte_termpos.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: TermPosGetSubterm()
//
//   Given a term and a position, return the denoted subterm.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

Term_p TermPosGetSubterm(Term_p term, TermPos_p pos)
{
   Term_p super;
   int    index;

   assert(term);

   if(PStackEmpty(pos))
   {
      return term;
   }
   assert(PStackGetSP(pos)>=2);
   
   index = PStackTopInt(pos);
   super = PStackBelowTopP(pos);
   
   assert(super->arity>index);
   
   return super->args[index];
}


/*-----------------------------------------------------------------------
//
// Function: TermPosFirstLIPosition()
//
//   Return the first subterm of term in leftmost-innermost order
//   and make pos the corresponding position.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

Term_p TermPosFirstLIPosition(Term_p term, TermPos_p pos)
{
   PStackReset(pos);

   while(term->arity)
   {
      PStackPushP(pos, term);
      PStackPushInt(pos, 0);
      term = term->args[0];
   }

   return term;
}

/*-----------------------------------------------------------------------
//
// Function:  TermPosNextLIPosition()
//
//   Given a term and a position, compute the next position (in
//   leftmost-innermost order) and return the corresponding term.
//
// Global Variables: -
//
// Side Effects    : Changes pos
//
/----------------------------------------------------------------------*/

Term_p TermPosNextLIPosition(Term_p term, TermPos_p pos)
{
   Term_p super;
   int    index;

   if(PStackEmpty(pos))
   {
      return NULL;
   }
   index = PStackPopInt(pos);
   super = PStackPopP(pos);
   if(index<super->arity-1)
   {
      index++;
      while(super->arity)
      {
	 PStackPushP(pos, super);
	 PStackPushInt(pos, index);
	 super = super->args[index];
	 index = 0;
      }
   }
   return super;	 
}


/*-----------------------------------------------------------------------
//
// Function: TermPosDebugPrint() 
//
//   Print a position in a term. If sig!=NULL, print terms, otherwise
//   print adddresses
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void TermPosDebugPrint(FILE* out, Sig_p sig, TermPos_p pos)
{
   PStackPointer i;

   fprintf(out, "# TermPos--\n");
   for(i=0; i<PStackGetSP(pos); i+=2)
   {
      fprintf(out, "# ");
      if(sig)
      {
	 TermPrint(out, PStackElementP(pos, i), sig, DEREF_NEVER);
	 fprintf(out, "...");
	 TermPrint(out, PStackElementP(pos, i), sig, DEREF_ALWAYS);
      }
      else
      {
	 fprintf(out, "<%d>", (int)PStackElementP(pos, i));
      }
      fprintf(out, " Subterm %ld\n", PStackElementInt(pos, i+1));
   }
   fprintf(out, "# --TermPos\n");
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/






