/*-----------------------------------------------------------------------

File  : cte_termtrees.h

Author: Stephan Schulz

Contents

  Functionality of term-top indexed AVL trees (I found that I can
  cleanly separate this from the termbank stuff).
  
  There are two sets of funktions for the manangment of term trees in
  CLIB: Funktions operating only on the top cell, and functions descending
  the term structure. Top level functions implement a conventional AVL
  tree with key f_code.masked_properties.entry_nos_of_args and are
  implemented here, recursive versions are in cte_termbanks.[hc]

Changes

<1> Thu Nov 27 19:12:51 MET 1997
    New

-----------------------------------------------------------------------*/

#ifndef CTE_TERMTREES

#define CTE_TERMTREES

#include <cte_termfunc.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/




/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

void   TermTreeFree(Term_p junk);
int    TermTopCompare(Term_p t1, Term_p t2, TermProperties prop_mask);
Term_p TermTreeFind(Term_p root, Term_p term, TermProperties prop_mask);
Term_p TermTreeInsert(Term_p *root, Term_p term, TermProperties prop_mask);
Term_p TermTreeExtract(Term_p *root, Term_p term, TermProperties prop_mask);
bool   TermTreeDelete(Term_p *root, Term_p term, TermProperties prop_mask);
void   TermTreeSetProp(Term_p root, TermProperties props);
void   TermTreeDelProp(Term_p root, TermProperties props);
long   TermTreeNodes(Term_p root);

AVL_TRAVERSE_DECLARATION(TermTree, Term_p)
#define TermTreeTraverseExit(stack) PStackFree(stack)


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





