/*-----------------------------------------------------------------------

File  : cte_termvars.h

Author: Stephan Schulz

Contents
 
  Functions for the management of shared variables.

Changes

<1> Tue Feb 24 15:52:12 MET 1998
    Rehacked with parts from the now obsolete cte_vartrans.h

-----------------------------------------------------------------------*/

#ifndef CTE_TERMVARS

#define CTE_TERMVARS

#include <clb_numtrees.h>
#include <cte_termtypes.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

/* Variable banks store information about variables. They contain two
   indices, one associating an external variable name with an internal
   term cell (and f_code, just because a StrTree can store two data
   items...), and one associate an f_code with a term cell. The first
   index is used for parsing and may be incomplete (i.e. not all
   variable cells will be indexed by a string), the second index
   should be complete. */

typedef struct varbankcell
{
   FunCode   v_count;      /* FunCode counter for new variables */
   StrTree_p ext_index;    /* Associate names and cells */
   NumTree_p f_code_index; /* Associate FunCodes and cells */
}VarBankCell, *VarBank_p;


/* We sometimes need to collect sets of variables and information
   about them, e.g. in checking the KBO variable condition. For small
   sets (e.g. variables in a term), this can be done rather
   efficiently in a hash. */


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define VarBankCellAlloc() (VarBankCell*)SizeMalloc(sizeof(VarBankCell))
#define VarBankCellFree(junk)         SizeFree(junk, sizeof(VarBankCell))

VarBank_p  VarBankAlloc();
void       VarBankFree(VarBank_p junk);
void       VarBankClearExtNames(VarBank_p bank);

Term_p VarBankFCodeFind(VarBank_p bank, FunCode f_code);
Term_p VarBankExtNameFind(VarBank_p bank, char* name);
Term_p VarBankFCodeAssertAlloc(VarBank_p bank, FunCode f_code);
Term_p VarBankGetFreshVar(VarBank_p bank);
Term_p VarBankExtNameAssertAlloc(VarBank_p bank, char* name);
#define VarBankGetVCount(bank) ((bank)->v_count)
#define VarBankSetVCount(bank,count) ((bank)->v_count = (count))
#define VarBankResetVCount(bank) ((bank)->v_count = 0)


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





