/*-----------------------------------------------------------------------

File  : cte_varhash.h

Author: Stephan Schulz

Contents
 
  Data structures for hashing and traversing variable occurences.

Changes

<1> Wed Jul 22 04:53:43 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CTE_VARHASH

#define CTE_VARHASH


#include <cte_termvars.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/


/* We sometimes need to collect sets of variables and information
   about them, e.g. in checking the KBO variable condition. For small
   sets (e.g. variables in a term), this can be done rather
   efficiently in a hash. */

#define VAR_HASH_SIZE 16  /* 2^n */
#define VAR_HASH_MASK (VAR_HASH_SIZE - 1)

typedef struct var_hash_entry_cell
{
   Term_p                     key;
   void*                      value;
   long                       val;
   struct var_hash_entry_cell *next;
}VarHashEntryCell, *VarHashEntry_p;

typedef struct var_hash_cell
{
   VarHashEntry_p hash[VAR_HASH_SIZE];
}VarHashCell, *VarHash_p;




/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


#define VarHashEntryCellAlloc() \
      (VarHashEntryCell*)SizeMalloc(sizeof(VarHashEntryCell))
#define VarHashEntryCellFree(junk) \
      SizeFree(junk, sizeof(VarHashEntryCell))

VarHashEntry_p VarHashEntryAlloc(Term_p var, long value);
void           VarHashEntryListFree(VarHashEntry_p list);

#define VarHashCellAlloc() \
      (VarHashCell*)SizeMalloc(sizeof(VarHashCell))
#define VarHashCellFree(junk) \
      SizeFree(junk, sizeof(VarHashCell))

VarHash_p      VarHashAlloc();
void           VarHashFree(VarHash_p junk);
int            VarHashFunction(Term_p var);
VarHashEntry_p VarHashListFind(VarHashEntry_p list, Term_p var);
void           VarHashAddValue(VarHash_p hash, Term_p var, long value);

void VarHashAddVarDistrib(VarHash_p hash, Term_p term, DerefType
                          deref, long add);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





